// 
// Copyright (C) 2006, NinjaTrader LLC <ninjatrader@ninjatrader.com>.
//
#region Using declarations
using System;
using System.Collections;
using System.Text;
#endregion

// This namespace holds all bars types. Do not change it.
namespace NinjaTrader.Data
{


	/// <summary>
	/// </summary>
	public class RenkoBarsType : BarsType
	{
		private static bool		registered		= Data.BarsType.Register(new RenkoBarsType());

		/// <summary>
		/// </summary>
		/// <param name="bars"></param>
		/// <param name="open"></param>
		/// <param name="high"></param>
		/// <param name="low"></param>
		/// <param name="close"></param>
		/// <param name="time"></param>
		/// <param name="volume"></param>
		/// <param name="isRealtime"></param>
		public override void Add(Data.Bars bars, double open, double high, double low, double close, DateTime time, int volume, bool isRealtime)
		{
			if (bars.Count == 0)
				AddBar(bars, open, high, low, close, time, volume);
			else
			{
				Data.Bar	bar			= (Bar) bars.Get(bars.Count - 1); 
				double		tickSize	= bars.Instrument.MasterInstrument.TickSize;
				double		rangeValue	= Math.Floor(10000000.0 * (double) bars.Period.Value * tickSize) / 10000000.0;

				if (bars.Instrument.MasterInstrument.Compare(close, bar.High + rangeValue) >= 0) 
				{
					bool	isFirstNewBar	= true;
                    double newBarOpen = bar.High;
                    double newClose = bar.High + rangeValue;
                                                                    
                    UpdateBar(bars, bar.Open, bar.High, bar.Low, bar.Close, time, 0);

                    do
                    {
                        AddBar(bars, newBarOpen, newClose, newBarOpen, newClose, time, isFirstNewBar ? volume : 0);
//                        UpdateBar(bars, newBarOpen, newClose, newBarOpen, newClose, time, isFirstNewBar ? volume : 0);
                        newBarOpen = newClose;
                        newClose += rangeValue;
                        isFirstNewBar = false;
                    }
                    while (bars.Instrument.MasterInstrument.Compare(close, newClose) >= 0);
				}
                else 
                   if (bars.Instrument.MasterInstrument.Compare(bar.Low - rangeValue, close) >= 0)
                {
                    bool isFirstNewBar = true;
                    double newClose = bar.Low - rangeValue;
                    double newBarOpen = bar.Low;
//                    if (open == bar.Low) UpdateBar(bars, bar.Low, bar.Low + rangeValue, bar.Low, bar.Low + rangeValue, time, 0);
                    UpdateBar(bars, bar.Open, bar.High, bar.Low, bar.Close, time, 0);

                    do
                    {
                        newClose = newBarOpen - rangeValue;
                        AddBar(bars, newBarOpen, newBarOpen, newClose, newClose, time, isFirstNewBar ? volume : 0);
                        newBarOpen = newClose;
                        newClose -= rangeValue;
                        isFirstNewBar = false;
                    }
                    while (bars.Instrument.MasterInstrument.Compare(newClose, close) >= 0);
               }
                else
                {
                    UpdateBar(bars, bar.Open, bar.High, bar.Low, bar.Close, time, volume);
                }
			}
		}

		/// <summary>
		/// </summary>
		public override PeriodType BuiltFrom
		{
			get { return PeriodType.Tick; }
		}

		/// <summary>
		/// </summary>
		/// <param name="time"></param>
		/// <returns></returns>
		public override string ChartDataBoxDate(DateTime time)
		{
			return time.ToString(Cbi.Globals.CurrentCulture.DateTimeFormat.ShortDatePattern);
		}

		/// <summary>
		/// </summary>
		/// <param name="chartControl"></param>
		/// <param name="time"></param>
		/// <returns></returns>
		public override string ChartLabel(NinjaTrader.Gui.Chart.ChartControl chartControl, DateTime time)
		{
			return time.ToString(chartControl.LabelFormatTick, Cbi.Globals.CurrentCulture);
		}

		/// <summary>
		/// </summary>
		/// <returns></returns>
		public override object Clone()
		{
			return new RenkoBarsType();
		}

		/// <summary>
		/// </summary>
		public override int DaysBack
		{
			get { return Gui.Chart.ChartData.DaysBackTick; }
		}

		/// <summary>
		/// </summary>
		public override int DefaultValue
		{ 
			get { return 4; }
		}

		/// <summary>
		/// </summary>
		public override string DisplayName
		{
			get { return PeriodType.ToString(); }
		}

		/// <summary>
		/// </summary>
		public override bool IsIntraday
		{
			get { return true; }
		}

		/// <summary>
		/// </summary>
		public override bool IsTimeBased
		{
			get { return false; }
		}

		/// <summary>
		/// </summary>
		public override int MaxLookBackDays
		{ 
			get { return 10;} 
		}

		/// <summary>
		/// </summary>
		public override int MaxValue
		{
			get { return -1; }
		}

		/// <summary>
		/// </summary>
		public override double GetPercentComplete(Data.Bars bars, DateTime now)
		{
			throw new ApplicationException("GetPercentComplete not supported in " + DisplayName);
		}

		/// <summary>
		/// </summary>
		public RenkoBarsType() : base(PeriodType.Renko)
		{
		}

		/// <summary>
		/// </summary>
		public override int SortOrder
		{
			get { return 13000; }
		}

		/// <summary>
		/// </summary>
		/// <param name="period"></param>
		/// <returns></returns>
		public override string ToString(Period period)
		{
			return "Renko " + period.Value.ToString() + " Ticks";
		}
	}
}
